/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package agent;

import util.*;
import java.awt.*;


/* ====================================================================== */
/** This class represents a user on the map.  A user is an agent which has
 *  a corresponding ipaq application that it sends data to via a port & socket.
 *  When a user changes location it sends its new gps position to its port so
 *  that the ipaq application can get it.
 *
 */
public class User extends Agent{

	/** The port to open a socket and send data to. */
  	private int port;



    /* ===================================================================== */
    /** This constructor initializes the user.
     *
     *  @param  n   The name of the user.
     *  @param  l   The starting location of this user on the map.
     *  @param  p   The port the ipaq gps receiver is listening on.
     *
     */
  	public User(String n, Point l, int p){

		super(n, l, new Position(), 75);
		port = p;

	} //Constructor


    /* ===================================================================== */
    /** This constructor initializes the user.
     *
     *  @param  l   The starting location of this user on the map.
     *  @param  p   The port the ipaq gps receiver is listening on.
     *
     */
  	public User(Point l, int p){

		super(l, new Position(), 75);
		port = p;

	} //Constructor



    /* ===================================================================== */
    /** This constructor initializes the user.
     *
     *  @param  p   The port the ipaq gps receiver is listening on.
     *
     */
  	public User(int p){

		super( new Point( 10, 10 ), new Position(), 75);
		port = p;

	} //Constructor




    /* ===================================================================== */
    /** This method returns the port this user is using.
     *
     *  @return  int   The port this user uses to send data to ipaq application.
     *
     */
	public int getPort(){

		return port;

	} //method: getPort




    /* ===================================================================== */
    /** This method sets the port to be used by this user.
     *
     *  @param  p   The port to set this user to use.
     *
     */
	public void setPort(int p){

		port = p;

	} //method: setPort




} //class: User

